// Nathan E. Moore
// March 23, 2004

#pragma once
#include "globals.h"
#include <stack>
#include <string>
#include <functional>
#include <boost\noncopyable.hpp>
#include <windows.h>

class NOVTABLE HwndInterface
{
public:
	// When displaying a recursive menu (TPM_RECURSE) we need to be able to notify the
	// window that it will get menu messages that are for a menu it doesn't know about
	// so ignore them.  The mode has to be set because it would be stupid to always
	// ignore the menu messages, as many things do nothing on a doContextMenu call
	enum Modes { NORMAL, CONTEXT_MENU };
	virtual void setMode(Modes m) = 0;
	virtual HWND getHwnd() = 0;
};

class MenuEntryInterface : public boost::noncopyable
{
private:
	// Never mess with the id!
	const unsigned int m_id;

	// May change to a random number
	static unsigned int idnext;
	static std::stack<unsigned int> idstack;
	// First id returned is NULL_ID + 1
	static unsigned int getNextId();

protected:
	// NULL_ID is used as the id for all dead items
	static const unsigned int NULL_ID = 0;

	MenuEntryInterface()
		:m_id(getNextId())
	{}

public:
	virtual ~MenuEntryInterface()
	{
		// return the id so that it can be used again
		idstack.push(m_id);
	}

	const unsigned int getId() const
	{
		return m_id;
	}

	virtual const std::wstring& getEntryText() const = 0;

	// Executes Item
	virtual void execute(HWND hwnd) = 0;

	// Displays Context Menu for the item
	virtual bool displayContextMenu(HwndInterface& hi) = 0;
};

class MenuEntryBase : public MenuEntryInterface
{
private:
	std::wstring m_entry;

protected:
	explicit MenuEntryBase(const std::wstring& txt)
		:m_entry(txt)
	{}

	MenuEntryBase()
	{}

	// only call from the constructor!
	// This is intended for use with automatic name detection
	void setEntryText(const std::wstring& str)
	{
		m_entry = str;
	}

public:
	const std::wstring& getEntryText() const
	{
		return m_entry;
	}
};

// for use with std::find
class EntrySearcher : public std::unary_function<MenuEntryPtr, bool>
{
private:
	unsigned int searchid;
public:
	EntrySearcher(unsigned int id)
		:searchid(id)
	{}

	bool operator()(const MenuEntryPtr& entryptr) const
	{
		return searchid == entryptr->getId();
	}
};
