// Nathan E. Moore
// April 8, 2004

#include "menuitem.h"
#include <shlwapi.h>
#include <shellapi.h>
#include <stdexcept>
#include <sstream>
#include <boost/scoped_array.hpp>

bool MenuItem::s_hasextensions = true;

MenuItem::MenuItem(const std::wstring& path, const std::wstring& arguments)
	:MenuEntryBase(std::wstring()),
	m_file(path),
	m_args(arguments)
{
	size_t length = path.length();
	boost::scoped_array<wchar_t> working(new wchar_t[length + 1]);
	path.copy(working.get(), length);
	working[length] = L'\0';

	// I should consider writing my own version of this and PathRemoveExtension
	// to reduce dependence on Shlwapi
	PathStripPath(working.get());

	if (!s_hasextensions)
		PathRemoveExtension(working.get());
	else // handle shortcut extensions
	{
		wchar_t* ext = working.get() + length - 4;
		if (lstrcmpi(ext, L".lnk") == 0)
			PathRemoveExtension(working.get());
	}

	setEntryText(working.get());
}

void MenuItem::execute(HWND hwnd)
{
	// find the directory the file is in so we can use that as the working directory
	// shortcuts ignore this, so it doesn't hurt
	boost::scoped_array<wchar_t> working;
	size_t endfolder = m_file.find_last_of(L'\\');
	if (endfolder != std::wstring::npos)
	{
		working.reset(new wchar_t[endfolder + 1]);
		m_file.copy(working.get(), endfolder);
		working[endfolder] = L'\0';
	}

	SHELLEXECUTEINFO seinfo;
	ZeroMemory(&seinfo, sizeof(seinfo));
	seinfo.cbSize = sizeof(seinfo);
	seinfo.fMask = SEE_MASK_DOENVSUBST;
	seinfo.hwnd = hwnd;
	seinfo.lpFile = m_file.c_str();
	seinfo.lpParameters = m_args.c_str();
	seinfo.lpDirectory = working.get();
	seinfo.nShow = SW_SHOW;
	if (!ShellExecuteEx(&seinfo))
	{
		std::ostringstream oss;
		oss << "ShellExecuteEx Failed. " << WinErrorMsg(GetLastError());
		throw std::runtime_error(oss.str());
	}
}
