// Nathan E. Moore
// March 23, 2004

#pragma once
#include "globals.h"
#include "menuentry.h"
#include "parser.h"
#include <string>
#include <atlbase.h>
#include <atlwin.h>
#include <atluser.h>

class HmenuWrapper : public HmenuInterface
{
public:
	WTL::CMenuHandle m_handle;

	HmenuWrapper(WTL::CMenuHandle& mh)
		:m_handle(mh)
	{}

	void insertEntry(const MenuEntryPtr& item, unsigned int location, bool column);
	void insertSubmenu(const MenuSubPtr& sub, unsigned int location, bool column);
	void insertSeparator(const unsigned int location, bool column);
	void insertDeadEntry(const std::wstring& txt, unsigned int location, bool column);
	bool deleteMenuElement(unsigned int location);
	void setColumnState(unsigned int location, bool column);
	bool getColumnState(unsigned int location);
};

class MenuSub : public MenuEntryInterface
{
private:
	static const unsigned int S_HEADER_LENGTH = 2;
	static long s_animation;
	static long s_position;
	static bool s_autocolumn;
	static bool s_header;

	bool m_autocolumn;
	ParserPtr m_parser;
	WTL::CMenu m_menu;
	WTL::CMenuHandle m_parent;

	explicit MenuSub(const ParserPtr& pip, bool columnson)
		:m_parser(pip),
		m_autocolumn(columnson)
	{
		m_menu.CreatePopupMenu();
		if (s_header)
		{
			HmenuWrapper hw(WTL::CMenuHandle(m_menu.m_hMenu));
			hw.insertDeadEntry(m_parser->getEntryText(), 0, false);
			m_menu.SetMenuDefaultItem(0, TRUE);
			hw.insertSeparator(1, false);
		}
	}

	void autocolumn(HWND hwnd);

public:
	// Search for a contained item
	MenuEntryPtr getEntry(unsigned int searchid) const
	{
		return m_parser->findEntry(searchid);
	}

	MenuSubPtr getSub(HMENU searchhmenu) const
	{
		return m_parser->findSubmenu(searchhmenu);
	}

	void beginDisplay(int x, int y, HWND hwnd);

	void display(HWND hwnd)
	{
		m_parser->predisplay(HmenuWrapper(WTL::CMenuHandle(m_menu)), (s_header ? S_HEADER_LENGTH : 0));
		autocolumn(hwnd);
	}

	void endDisplay()
	{
		m_parser->postdisplay(HmenuWrapper(WTL::CMenuHandle(m_menu)), (s_header ? S_HEADER_LENGTH : 0));
	}

	void execute(HWND)
	{}

	bool displayContextMenu(HwndInterface& hi)
	{
		return m_parser->doContextMenu(hi);
	}

	const std::wstring& getEntryText() const
	{
		return m_parser->getEntryText();
	}

	friend class SubSearcher;
	friend class HmenuWrapper;

	static void setAnimation(const char* horzmode, const char* vertmode);
	static void setPosition(const char* horzmode, const char* vertmode);

	static void enableAutoColumn(bool flag)
	{
		s_autocolumn = flag;
	}

	static void enableHeaders(bool flag)
	{
		s_header = flag;
	}

	static MenuSubPtr createMenuSub(const ParserPtr& pip, bool columnson)
	{
		MenuSubPtr temp(new MenuSub(pip, columnson));
		return temp;
	}
};

class SubSearcher : public std::unary_function<boost::shared_ptr<MenuSub>, bool>
{
private:
	HMENU searchmenu;
public:
	SubSearcher(HMENU menu)
		:searchmenu(menu)
	{}

	bool operator()(const boost::shared_ptr<MenuSub>& subptr) const
	{
		return searchmenu == subptr->m_menu.m_hMenu;
	}
};
