// Nathan E. Moore
// March 30, 2004
// This is designed to be used with CombiningParser, and not as a freestanding menu

#pragma once
#include "parser.h"
#include "menuentry.h"
#include <string>

namespace date_impl
{
	class DateEntry;
	typedef boost::shared_ptr<DateEntry> DateEntryPtr;

	class DateEntry : public MenuEntryBase
	{
	private:
		static const std::wstring S_INET_TIME_PATTERN;
		static const size_t S_FORMATED_BUFFER_SIZE;
		std::wstring m_pattern;

		DateEntry(const std::wstring& pattern)
			:m_pattern(pattern)
		{}

	public:
		bool displayContextMenu(HwndInterface&)
		{
			return false;
		}

		void update();
		void execute(HWND hwnd);

		static int getInetTime(__time64_t now);
		static std::wstring formatTime(const std::wstring& pattern, tm *timedata);

		static DateEntryPtr createDateEntry(const std::wstring& pattern)
		{
			DateEntryPtr temp(new DateEntry(pattern));
			return temp;
		}
	};
}

class DateParser : public DynamicParserBase
{
private:
	date_impl::DateEntryPtr m_entry;

	DateParser(const std::wstring& pattern)
		:m_entry(date_impl::DateEntry::createDateEntry(pattern))
	{}

public:
	MenuEntryPtr findEntry(unsigned int id) const
	{
		if (id == m_entry->getId())
			return boost::static_pointer_cast<MenuEntryPtr::element_type>(m_entry);
		else
			return MenuEntryPtr();
	}

	unsigned int parse(HmenuInterface& menu, unsigned int start)
	{
		m_entry->update();
		menu.insertEntry(m_entry, start);
		return 1;
	}

	MenuSubPtr findSubmenu(HMENU) const
	{
		return MenuSubPtr();
	}

	bool doContextMenu(HwndInterface&)
	{
		return false;
	}

	static ParserPtr createDateParser(const std::wstring& pattern)
	{
		ParserPtr temp(new DateParser(pattern));
		return temp;
	}
};
