// Nathan E. Moore
// April, 2, 2004

#include "folderparser.h"
#include "menusub.h"
#include "menuitem.h"
#include <sstream>

// this could still use some more work, paths like C:\*\folder\ will mess up
void fp_impl::Path::validate()
{
	// 1) Determine if a pattern is part of this->path
	// 2) if so seperate it out into this->file
	// 3) otherwise backslashify this->path

	size_t location = m_path.find_last_of(L'*');
	if (location == std::wstring::npos)
	{
		location = m_path.find_last_of(L'?');
		if (location == std::wstring::npos)
		{
			m_file = L"*";

			// Do PathAddBackslash without the messy conversion to wchar_t
			if (*m_path.rbegin() != L'\\')
				m_path.append(L"\\");

			return ;
		}
	}
	location = m_path.find_last_of(L'\\');
	if (location != std::wstring::npos)	// Should be redundent
	{
		location++;
		m_file = m_path.substr(location, m_path.length() - location);
		m_path = m_path.substr(0, location);
	}
}

void fp_impl::parseFolder(const Path& parsepath, MergeableInterface& mi)
{
	WIN32_FIND_DATA FindFileData;
	HANDLE hfind = FindFirstFile(parsepath.getSearchPath().c_str(), &FindFileData);
	if (hfind == INVALID_HANDLE_VALUE)
	{
		std::ostringstream oss;
		oss << "FindFirstFile returned an Invalid Handle. " << WinErrorMsg(GetLastError()) << " " LOCATION;
		throw std::runtime_error(oss.str());
	}

	std::wstring filename;
	std::wstring completepath;
	do
	{
		if (!(lstrcmp(FindFileData.cFileName, L"..") == 0 || lstrcmp(FindFileData.cFileName, L".") == 0 ||
		        (FindFileData.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) == FILE_ATTRIBUTE_HIDDEN))
		{
			filename = FindFileData.cFileName;
			completepath = parsepath.m_path;
			completepath += filename;
			if ((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
				mi.addSubMenu(filename, completepath);
			else
				mi.addEntry(MenuItem::createMenuItem(completepath, std::wstring()));
		}
	}
	while (FindNextFile(hfind, &FindFileData));
	FindClose(hfind);
}
