// Nathan E. Moore
// April 3, 2003

#include "globals.h"
#include <sstream>

// needed by toUnicodeNoThrow
void nullDeleter(wchar_t*) throw();

void getMonitorArea(POINT* pt, RECT* area) throw()
{
	HMONITOR hmon;
	hmon = MonitorFromPoint(*pt, MONITOR_DEFAULTTONEAREST);
	MONITORINFO moni;
	moni.cbSize = sizeof(moni);
	GetMonitorInfo(hmon, &moni);
	*area = moni.rcMonitor;
}

boost::shared_array<wchar_t> toUnicode(const char* astr)
{
	size_t astrsize = strlen(astr);
	int wstrsize = MultiByteToWideChar(CP_THREAD_ACP, 0, astr + 1, astrsize, NULL, 0);
	boost::shared_array<wchar_t>wstr(new wchar_t[wstrsize + 1]);
	if (!MultiByteToWideChar(CP_ACP, 0, astr, astrsize + 1, wstr.get(), wstrsize + 1))
	{
		std::ostringstream oss;
		oss << "Error converting \"" << astr << "\" to unicode. " << WinErrorMsg(GetLastError());
		throw std::runtime_error(oss.str());
	}
	return wstr;
}

boost::shared_array<wchar_t> toUnicodeNoThrow(const char* astr) throw()
{
	try
	{
		return toUnicode(astr);
	}
	catch (std::exception& e)
	{
		LSLOG(e.what());
		return boost::shared_array<wchar_t>(L"ERROR!", nullDeleter);
	}
}

void nullDeleter(wchar_t*) throw()
{}
