// Nathan E. Moore
// March 23, 2004

#include "manualparser.h"
#include "menuentry.h"

unsigned int ManualParser::parse(HmenuInterface& menu, unsigned int start)
{
	EntryVector::iterator entryiter = m_entries.begin();
	SubVector::iterator subiter = m_subs.begin();
	DeadEntryList::iterator deaditer = m_deadentries.begin();
	unsigned int location = start;
	for (PatternList::iterator i = m_pattern.begin(); i != m_pattern.end(); ++i, ++location)
	{
		bool column = S_NEW_COLUMN == ((*i).second & S_NEW_COLUMN);
		switch ((*i).first)
		{
		case S_ENTRY:
			_ASSERTE(m_entries.end() != entryiter);
			menu.insertEntry(*entryiter, location, column);
			++entryiter;
			break;
		case S_SUBMENU:
			_ASSERTE(m_subs.end() != subiter);
			menu.insertSubmenu(*subiter, location, column);
			++subiter;
			break;
		case S_SEPARATOR:
			menu.insertSeparator(location, column);
			break;
		case S_DEAD_ENTRY:
			_ASSERTE(m_deadentries.end() != deaditer);
			menu.insertDeadEntry(*deaditer, location, column);
			++deaditer;
			break;
		default:
			// explode
			_ASSERT(false);
			DEBUG_OUT("INVALID Pattern symbol found.");
			break;
		}
	}
	_ASSERTE((m_entries.end() == entryiter) && (m_subs.end() == subiter) && (m_deadentries.end() == deaditer));

	// get rid of the pattern as we don't need it any more
	m_pattern.clear();
	// nor do we need the deadentry strings
	m_deadentries.clear();

	return location - start;
}

MenuEntryPtr ManualParser::findEntry(unsigned int id) const
{
	// in here
	EntryVector::const_iterator iter = std::find_if(m_entries.begin(), m_entries.end(), EntrySearcher(id));
	if (iter != m_entries.end())
		return *iter;

	// in a submenu
	// TODO: get rid of the recursion
	for (SubVector::const_iterator i = m_subs.begin(), end = m_subs.end(); i != end; ++i)
	{
		MenuEntryPtr item = (*i)->getEntry(id);
		if (item)
			return item;
	}

	// not in this menu hierarchy
	return MenuEntryPtr();
}

MenuSubPtr ManualParser::findSubmenu(HMENU hmenu) const
{
	// in here
	SubVector::const_iterator iter = std::find_if(m_subs.begin(), m_subs.end(), SubSearcher(hmenu));
	if (iter != m_subs.end())
		return *iter;

	// in a submenu
	// TODO: get rid of the recursion
	for (SubVector::const_iterator i = m_subs.begin(), end = m_subs.end(); i != end; ++i)
	{
		MenuSubPtr sub = (*i)->getSub(hmenu);
		if (sub)
			return sub;
	}

	// not in this menu hierarchy
	return MenuSubPtr();
}
