// Nathan Moore
// April 11, 2004

#pragma once
#include "menusub.h"
#include "globals.h"
#include "../nem_utils/ls.h"

#include <string>
#include <utility>
#include <functional>
#include <hash_map>
#include <boost/shared_ptr.hpp>
#include <boost/noncopyable.hpp>
#include <atlbase.h>
#include <atlwin.h>
#include <strsafe.h>

extern "C" void onBangCmdFunction(HWND, const char* bang, const char* args);

typedef CWinTraits < WS_POPUP | WS_CLIPCHILDREN, WS_EX_TOOLWINDOW | WS_EX_NOPARENTNOTIFY > MenuWinTraits;

template <class T>
class BangableMenu :
	public CMessageMap,
	public HwndInterface
{
private:
	static const UINT_PTR TIMER_ID = 1;
	MenuSubPtr m_menu;
	bool m_normalmode;

	LRESULT onEnterMenuLoop(UINT, WPARAM, LPARAM, BOOL&)
	{
		// Kill the timer so it doesn't destroy the dynamic menu's while in use
		static_cast<T*>(this)->KillTimer(TIMER_ID);
		// Get Focus so that keyboard input will work
		SwitchToThisWindow(static_cast<T*>(this)->m_hWnd, FALSE);
		return 0;
	}

	LRESULT onInitMenuPopup(UINT, WPARAM wparam, LPARAM, BOOL&)
	{
		try
		{
			MenuSubPtr msptr;
			if (SubSearcher(reinterpret_cast<HMENU>(wparam))(m_menu))
				msptr = m_menu;
			else
				msptr = m_menu->getSub(reinterpret_cast<HMENU>(wparam));

			if (msptr)
				msptr->display(static_cast<T*>(this)->m_hWnd);
		}
		catch (std::exception& e)
		{
			LSLOG(lslog::error << "Exception caught after onInitMenuPopup. " << e.what());
		}
		return 0;
	}

	LRESULT onCommand(UINT, WPARAM wparam, LPARAM, BOOL&)
	{
		if (HIWORD(wparam) == 0)	// Is from a menu
		{
			if (MenuEntryPtr meptr = m_menu->getEntry(LOWORD(wparam)))
			{
				try
				{
					meptr->execute(static_cast<T*>(this)->m_hWnd);
				}
				catch (std::exception& e)
				{
					LSLOG(lslog::error << "Execution Failed. " << e.what());
				}
			}
		}
		return 0;
	}

	LRESULT onMenuRButtonUp(UINT, WPARAM wparam, LPARAM lparam, BOOL&)
	{
		MENUITEMINFO info;
		info.cbSize = sizeof(info);
		info.fMask = MIIM_FTYPE | MIIM_ID | MIIM_STATE | MIIM_SUBMENU;
		if (!GetMenuItemInfo(reinterpret_cast<HMENU>(lparam), wparam, true, &info))
		{
			LSLOG(lslog::debug << "GetMenuItemInfo Failed. " << WinErrorMsg(GetLastError()));
			return 0;
		}

		// disabled is how deaditems are determined.
		if (!(info.fType & MFT_SEPARATOR) && !(info.fState & MFS_DISABLED))
		{
			try
			{
				MenuEntryPtr selectedItem;
				if (info.hSubMenu == NULL)
					selectedItem = m_menu->getEntry(info.wID);
				else
					selectedItem = m_menu->getSub(info.hSubMenu);

				// Verify that we really found the item
				if (selectedItem)
				{
					if (selectedItem->displayContextMenu(*this))
						EndMenu();
				}
			}
			catch (std::exception& e)
			{
				LSLOG(lslog::error << "Exception caught after displayContextMenu(). " << e.what());
				EndMenu();
			}
		}
		return 0;
	}

	LRESULT onTimer(UINT, WPARAM, LPARAM, BOOL&)
	{
		static_cast<T*>(this)->KillTimer(TIMER_ID);
		try
		{
			m_menu->endDisplay();
		}
		catch (std::exception& e)
		{
			LSLOG(lslog::error << "Exception caught in onTimer. " << e.what());
		}
		return 0;
	}

	LRESULT onExitMenuLoop(UINT, WPARAM, LPARAM, BOOL&)
	{
		// There is no fall back if the a timer cannot be had, without the timer
		// everything still works, just the dynamic menus are not dynamic without
		// it when no item is selected
		if (!static_cast<T*>(this)->SetTimer(TIMER_ID, 250))
			LSLOG(lslog::debug << "SetTimerFailed. " << WinErrorMsg(GetLastError()));
		return 0;
	}

protected:
	explicit BangableMenu(MenuSubPtr subptr)
		:m_menu(subptr)
	{
		// setup m_normalmode
		setMode(NORMAL);
	}

public:

	virtual ~BangableMenu()
	{}

	void onBang(const char* args)
	{
		POINT pt;
		GetCursorPos(&pt);
		// the argument handling code needs to be rewritten
		/*
				// Handle any params that come with the bang
				char sztoken1[MAX_RC_STRING];
				char sztoken2[MAX_RC_STRING];
				char* rgsztokens[] = { sztoken1, sztoken2 };
		 
				int ntokens = LCTokenize(args, rgsztokens, 2, NULL);
				// There are actually two more possible args, however they cannot be implemented with menu
		 
				// this huge comparison is because pszArguments will always return something
				// it should IMO return NULL, "", or have another arg telling you how long the args are
				// if there are no params, but it doesn't
				// even this will not catch every possiblity, but anything that is incorrect that gets past
				// this will have the menu display at 0,0 
				// this is still a problem here, a exception is thrown
				if (ntokens >= 2)
				{
		 
					if (isdigit(sztoken1[0]) || sztoken1[0] == '-' || _strnicmp(sztoken1, "X=", 2) == 0)
					{
						int x, y;
						char* ptok1 = sztoken1;
						char* ptok2 = sztoken2;
		 
						if (_strnicmp(sztoken1, "X=", 2) == 0)
						{
							x = atoi(sztoken1 + 2);
							ptok1 = sztoken1 +  2;
						}
						else
							x = atoi(sztoken1);
		 
						if (_strnicmp(sztoken2, "y=", 2) == 0)
						{
							y = atoi(sztoken2 + 2);
							ptok2 = sztoken2 +  2;
						}
						else
							y = atoi(sztoken2);
		 
						if (x != 0 && ptok1[0] != '0')
							pt.x = CONVERT_COORDINATE_X(x);
						if (y != 0 && ptok2[0] != '0')
							pt.y = CONVERT_COORDINATE_Y(y);
					}	
				}
		*/
		m_menu->beginDisplay(pt.x, pt.y, static_cast<T*>(this)->m_hWnd);
	}

	// this is the HwndInterface stuff
	void setMode(Modes m)
	{
		switch (m)
		{
		case NORMAL:
			m_normalmode = true;
			break;
		case CONTEXT_MENU:
			m_normalmode = false;
			break;
		default:
			_ASSERT(false);
		}
	}

	HWND getHwnd()
	{
		return static_cast<T*>(this)->m_hWnd;
	}


	BEGIN_MSG_MAP(BangableMenu)
		if (m_normalmode)
		{
			MESSAGE_HANDLER(WM_ENTERMENULOOP, onEnterMenuLoop)
			MESSAGE_HANDLER(WM_INITMENUPOPUP, onInitMenuPopup)
			MESSAGE_HANDLER(WM_COMMAND, onCommand)
			MESSAGE_HANDLER(WM_MENURBUTTONUP, onMenuRButtonUp)
			MESSAGE_HANDLER(WM_EXITMENULOOP, onExitMenuLoop)
			MESSAGE_HANDLER(WM_TIMER, onTimer)
		}
	END_MSG_MAP()
};

class BangableMenuWindow :
	public CWindowImpl<BangableMenuWindow, CWindow, MenuWinTraits>,
	public BangableMenu<BangableMenuWindow>
{
public:
	explicit BangableMenuWindow(MenuSubPtr subptr)
		:BangableMenu<BangableMenuWindow>(subptr)
	{}

	DECLARE_WND_CLASS(G_WMODULE_NAME)

	BEGIN_MSG_MAP(BangableMenuWindow)
		CHAIN_MSG_MAP(BangableMenu<BangableMenuWindow>)
	END_MSG_MAP()
};

// This class controls the life time of all the windows, and handles intial bang routing.
// eventually this is were the extra bangs will be.
class Menu :
	public CWindowImpl<Menu, CWindow, MenuWinTraits>,
	public boost::noncopyable
{
private:
	// the hash<std::string> is the same as the hash<const char*> reimplemented
	// with std::string stuff same as the java.lang.String.hashCode impl with 5 
	// instead of 31
	typedef boost::shared_ptr<BangableMenuWindow> BangableMenuWindowPtr;
	typedef std::hash_map<std::string, BangableMenuWindowPtr> BangMap;
	BangMap m_bangs;
	HWND m_parenthwnd;

	LRESULT onGetRevId(UINT, WPARAM, LPARAM lparam, BOOL&)
	{
		// the buffer is MAX_LINE_LENGHT (4096), so we don't overflow it here.
		// according to ls source we should assume a size of 64.
		// that is way to fucking big. The buffer size should be passed in
		// WPARAM, and should be between 64 and 512 chars in length.
		// oh but wait wparam is used for some flag that no one ever checks!!
		char* buf = reinterpret_cast<char*>(lparam);

		// Goodbye type safety
		StringCchPrintfA(buf, 64, "%s %s", G_MODULE_NAME, G_MODULE_VERSION);
		size_t length;
		if (SUCCEEDED(StringCchLengthA(buf, 64, &length)))
			return length;
		else
		{
			*buf = '\0';
			return 0;
		}
	}

	LRESULT onLMPopup(UINT, WPARAM, LPARAM, BOOL&)
	{
		onBangCmd("!popup", NULL);
		return 0;
	}

	LRESULT onLMHidePopup(UINT, WPARAM, LPARAM, BOOL&)
	{
		EndMenu();
		return 0;
	}

public:
	explicit Menu(HWND parentwnd);
	~Menu();

	DECLARE_WND_CLASS(G_WMODULE_NAME)

	BEGIN_MSG_MAP(Menu)
		MESSAGE_HANDLER(LM_POPUP, onLMPopup)
		MESSAGE_HANDLER(LM_HIDEPOPUP, onLMHidePopup)
		MESSAGE_HANDLER(LM_GETREVID, onGetRevId)
	END_MSG_MAP()

	void onBangCmd(const char* name, const char* args)
	{
		BangMap::iterator bangwnd = m_bangs.find(name);
		if (m_bangs.end() != bangwnd)
			(*bangwnd).second->onBang(args);
	}

	void addBang(const std::string& cmdstr, const MenuSubPtr& popup);
};
