// Nathan E. Moore
// March 23, 2004

#include "tasksparser.h"
#include <boost/scoped_array.hpp>
#include <atlbase.h>
#include <atlwin.h>
#include <atluser.h>

#ifndef WS_EX_NOACTIVATE
	#define WS_EX_NOACTIVATE 0x08000000L
#endif

class MenuWindow : public MenuEntryBase
{
private:
	HWND m_taskhwnd;	// Never needs to be validated because this is dynamic menu item

	MenuWindow(const std::wstring& txt, HWND handle)
		:MenuEntryBase(txt),
		m_taskhwnd(handle)
	{}

public:
	void execute(HWND)
	{
		if(IsIconic(m_taskhwnd))
			ShowWindow(m_taskhwnd, SW_RESTORE);
		else
            SwitchToThisWindow(m_taskhwnd, FALSE);
	}

	bool displayContextMenu(HwndInterface& hi);
	
	static MenuEntryPtr createMenuWindow(const std::wstring& txt, HWND handle)
	{
		MenuEntryPtr temp(new MenuWindow(txt, handle));
		return temp;
	}
};

BOOL TasksParser::enumWindowsProc(HWND hwnd, LPARAM lparam)
{
	// Validation
	LONG_PTR style = GetWindowLongPtr(hwnd, GWL_STYLE);
	if ((style & WS_CHILD) == WS_CHILD || (style & WS_VISIBLE) != WS_VISIBLE)
		return TRUE;

	style = GetWindowLongPtr(hwnd, GWL_EXSTYLE);
	if ((style & WS_EX_TOOLWINDOW) == WS_EX_TOOLWINDOW || ( (style & WS_EX_NOACTIVATE) == WS_EX_NOACTIVATE && (style & WS_EX_APPWINDOW) != WS_EX_APPWINDOW ))
		return TRUE;

	style = GetWindowLongPtr(hwnd, GWL_HWNDPARENT);
	if (style != 0)
		return TRUE;

	// Creating a new menu Item;
	TasksParseData* data = reinterpret_cast<TasksParseData*>(lparam);

	int bufsize = GetWindowTextLength(hwnd) + 1;
	boost::scoped_array<wchar_t> buffer(new wchar_t[bufsize]);
	GetWindowText(hwnd, buffer.get(), bufsize);

	MenuEntryPtr eptr = MenuWindow::createMenuWindow(std::wstring(buffer.get()), hwnd);
	data->parser->m_wnds.push_front(eptr);
	data->menu->insertEntry(eptr, data->location);
	++(data->location);

	return TRUE;
}

bool MenuWindow::displayContextMenu(HwndInterface& hi)
{
	// Known Problem: The PowerMenu Additions do not work
	// Although the winamp additions do, possibly an error in powermenu

	// We don't free the HMENU from GetSystemMenu as then it doesn't work the next time.
	WTL::CMenuHandle sysmenu(::GetSystemMenu(m_taskhwnd, FALSE));
	if (!sysmenu.IsMenu())
		return false;

	WINDOWPLACEMENT wp;
	ZeroMemory(&wp, sizeof(WINDOWPLACEMENT));
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(m_taskhwnd, &wp);

	sysmenu.EnableMenuItem(SC_RESTORE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED));
	sysmenu.EnableMenuItem(SC_MOVE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED));
	sysmenu.EnableMenuItem(SC_SIZE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED));
	sysmenu.EnableMenuItem(SC_MINIMIZE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED));
	sysmenu.EnableMenuItem(SC_MAXIMIZE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED));

	SendMessage(m_taskhwnd, WM_INITMENUPOPUP, reinterpret_cast<WPARAM>(sysmenu.m_hMenu), MAKELPARAM(0, TRUE));
	SendMessage(m_taskhwnd, WM_INITMENU, reinterpret_cast<WPARAM>(sysmenu.m_hMenu), 0);

	POINT pt;
	GetCursorPos(&pt);

	hi.setMode(HwndInterface::CONTEXT_MENU);
	int cmd = sysmenu.TrackPopupMenu(TPM_RECURSE | TPM_RETURNCMD | TPM_RIGHTBUTTON, pt.x, pt.y, hi.getHwnd());
	hi.setMode(HwndInterface::NORMAL);

	if (!cmd)
		return false;
	else if (cmd != SC_MINIMIZE && cmd != SC_CLOSE)
		execute(hi.getHwnd());
	SendMessage(m_taskhwnd, WM_SYSCOMMAND, (WPARAM) cmd, MAKELPARAM(pt.x, pt.y));
	return true;
}
