// Nathan E. Moore
// March 20, 2004

// defines template classes for an ostream to a function taking a char* (or wchar_t*)
// and typedefs for output to OutputDebugString

// TODO: the debugbuf_base class needs to have a limit on how much it caches before writing
//       at the moment it will cache an infinite amount

#pragma once
#include <sstream>
#include <ostream>
#include <windows.h>

namespace util
{
	template <typename Elem, typename Funct>
	class debugbuf_base : public std::basic_stringbuf<Elem>
	{
	private:
		Funct outfn;

	protected:
		int sync()
		{
			outfn(str().c_str());
			str(std::basic_string<Elem>());
			return 0;
		}

	public:
		virtual ~debugbuf_base()
		{
			sync();
		}
	};

	template <typename Elem, typename Funct>
	class debugostream_base : public std::basic_ostream<Elem>
	{
	public:
		debugostream_base()
				: std::basic_ostream<Elem>(new debugbuf_base<Elem, Funct>())
		{}

		virtual ~debugostream_base()
		{
			delete rdbuf();
		}
	};

	template <typename Elem>
	struct OutputDebugStringFunctor
	{};

	template <>
	struct OutputDebugStringFunctor<char>
	{
		inline void operator()(const char* str) const
		{
			::OutputDebugStringA(str);
		}
	};

	template <>
	struct OutputDebugStringFunctor<wchar_t>
	{
		inline void operator()(const wchar_t* str) const
		{
			::OutputDebugStringW(str);
		}
	};

	typedef debugostream_base<char, OutputDebugStringFunctor<char> > debugout;
	typedef debugostream_base<wchar_t, OutputDebugStringFunctor<wchar_t> > wdebugout;
}
