// Nathan E. Moore
// Feb. 11, 2004
// This needs RTTI enabled

// TODO: fix this to use debugstream.h and dump the RTTI
#pragma once
#include <sstream>
#include <string>
#include "ls.h"

namespace lslog
{
	class LSLogBuf : public std::stringbuf
	{
	private:
		std::string m_module;
		int m_loglevel;

	public:
		LSLogBuf(const std::string& module, int loglevel)
				: m_module(module),
				m_loglevel(loglevel)
		{}

		virtual ~LSLogBuf()
		{
			write();
		}

		void setLogLevel(int level)
		{
			m_loglevel = level;
		}

		void write()
		{
			std::string msg(str());
			if (!msg.empty())
			{
				LSLog(m_loglevel, m_module.c_str(), msg.c_str());
				str(std::string());
			}
		}
	};

	inline std::ostream& error(std::ostream& os)
	{
		LSLogBuf* buf = dynamic_cast<LSLogBuf*>(os.rdbuf());
		if (buf)
			buf->setLogLevel(LOG_ERROR);
		return os;
	}

	inline std::ostream& warning(std::ostream& os)
	{
		LSLogBuf* buf = dynamic_cast<LSLogBuf*>(os.rdbuf());
		if (buf)
			buf->setLogLevel(LOG_WARNING);
		return os;
	}

	inline std::ostream& notice(std::ostream& os)
	{
		LSLogBuf* buf = dynamic_cast<LSLogBuf*>(os.rdbuf());
		if (buf)
			buf->setLogLevel(LOG_NOTICE);
		return os;
	}

	inline std::ostream& debug(std::ostream& os)
	{
		LSLogBuf* buf = dynamic_cast<LSLogBuf*>(os.rdbuf());
		if (buf)
			buf->setLogLevel(LOG_DEBUG);
		return os;
	}

	inline std::ostream& endmsg(std::ostream& os)
	{
		os.flush();
		LSLogBuf* buf = dynamic_cast<LSLogBuf*>(os.rdbuf());
		if (buf)
			buf->write();
		return os;
	}

	class LSLogStream : public std::ostream
	{
	public:
		LSLogStream(const std::string& module, int loglevel)
				: std::ostream(new LSLogBuf(module, loglevel))
		{}

		~LSLogStream()
		{
			endmsg(*this);
			delete rdbuf();
		}
	};
}
